#!/usr/bin/python
from math import *
import numpy as np
import matplotlib.pyplot as plt
from random import random
from scipy.optimize import root
import sys

# Define model constants
L=1.0
R=3.0
omega=5.0
rho=1.0
n=20
pts=251

# Integration points
xx=np.linspace(0,L,pts)

# Function to give rope position and s-derivative
def pos(x,b):
    y=0
    dy=0
    for k in range(1,n+1):
        th=pi*k*x/L
        y+=b[k]*sin(th)
        dy+=b[k]*k*cos(th)
    dy*=pi/L
    return (y,dy)

# Function to be minimized
def phi(b):
    f=0
    for i in range(pts):
        (y,dy)=pos(xx[i],b)
        dfac=sqrt(1+dy*dy)
        T=dfac*(rho*y*y*omega*omega+b[0])
        if i==0 or i==pts-1:
            f+=T
        else:
            f+=2*T
    return f*L/(2*(pts-1))-b[0]*R

# Gradient
def grad_phi(b):
    f=np.zeros((n+1))

    for i in range(pts):
        
        # Calculate (x,y) and (x_s,y_s), as well as factor for this
        # contribution in the trapezoidal rule
        x=xx[i]
        (y,dy)=pos(x,b)
        if i==0 or i==pts-1:
            fac=1
        else:
            fac=2
        
        # Do intermediate calculations required to compute the
        # gradient
        dfac=sqrt(1+dy*dy)
        fac1=fac*2*rho*omega*omega*dfac*y
        fac2=fac*dy/dfac*(rho*omega*omega*y*y+b[0])*pi/L
    
        # Assemble the components of the gradient
        f[0]+=fac*dfac
        for k in range(1,n+1):
            th=pi*k*x/L
            f[k]+=fac1*sin(th)
            f[k]+=fac2*k*cos(th)
    f*=L/(2*(pts-1))
    f[0]-=R

    return f

# Set initial guess 
b_init=np.zeros((n+1))
b_init[1]=1.3

# Do Levenberg-Marquardt algorithm
sol=root(grad_phi,b_init,jac=False,method='lm')

# Uncomment these lines to print diagnostic information
print("Predicted location:",sol.x)
print("grad(phi):",grad_phi(sol.x))
print("phi:",phi(sol.x))

# Print the initial and final shapes of the ropes
for i in range(pts):
    x=xx[i]
    (y,dy)=pos(x,sol.x)
    (y2,dy2)=pos(x,b_init)
    print(x,y,dy,y2,dy2)
