#!/usr/bin/python
import numpy as np
import scipy.sparse as sp
import scipy.sparse.linalg as spl
from math import *

# Set grid resolution and create second derivative matrix. The number of grid
# points should be divisible by four, to ensure that there are points at x=0
# and x=6 for the integration.
n=1920
h=24/n
x=np.linspace(-12+h,12-h,n-1)
f=1/(h*h)
D2=sp.diags([f,-2*f,f],[-1,0,1],shape=(n-1,n-1))

# Create array of weights for Simpson's integration on the whole interval
f=h/6
sf=np.empty(n-1)
for k in range(n-1):
    sf[k]=f*(2,4)[k%2==0]

# Create array of weights for Simpson's integration on the [0,6] interval
s06=np.empty(n//4+1)
for k in range(1,n//4):
    s06[k]=f*(4,2)[k%2==0]
s06[0]=f
s06[n//4]=f

# Loop over the four potentials. Case zero is x^2/10, shown as an example
# in the homework.
for k in range(0,4):
    print("Case",k,"\n======")

    # Compute the potential for this case
    if k==0:
        v=0.1*x*x
    elif k==1:
        v=np.abs(x)
    elif k==2:
        v=1.3+x*(0.125+x*(-1/18.+0.0012*x*x))
    else:
        v=7*np.abs(np.abs(np.abs(x)-1)-1)

    # Assemble matrix and compute the five smallest eigenvalues and
    # corresponding eigenvectors
    A=-D2+sp.diags([v],[0])
    [lam,vec]=spl.eigs(A,k=5,which='SM')
    
    # Loop over the five eigenvalues
    for l in range(5):

        # Calculate probability that the particle for this eigenfunction
        # is in the range [0,6]
        mod=np.abs(vec[:,l])
        mod*=mod
        prob=np.dot(s06,mod[n//2-1:3*n//4])/np.dot(sf,mod)

        # Print the eigenvalue and the probability
        print("Eigenvalue",l+1,"=",np.real(lam[l]),"   P(0<x<6) =",prob)
    print()
    
    # Write the eigenvectors to a file
    q=open("qfunc"+str(k)+".dat","w")
    q.write("-12 0 0 0 0 0\n");
    for l in range(n-1):
        q.write(str(x[l])+" "+" ".join([str(np.real(vec[l,i])) for i in range(5)])+"\n")
    q.write("12 0 0 0 0 0\n");
    q.close()
